/*
 * Batch.java
 *
 * Created on April 27, 2007, 6:09 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package medusa.batchoperations;

import java.io.File;
import java.io.IOException;
import medusa.DataFormatException;
import medusa.dataio.DataLoader;
import medusa.display.EditableGraphPanel;
import medusa.display.FRspring;
import medusa.graph.Graph;

/**
 * This is an example of how to use Medusa in a batch mode. 
 * Useful for layong out lots of graphs or generating images.
 *
 * @author shooper
 */
public class BatchOperations {
    DataLoader dl;
    /** Creates a new instance of Batch */
    public BatchOperations() {
        dl=new DataLoader(1,1);
    }
    
    // 1. convert all files which follow the tabbed rule
    // to medusa files
    public void convertTabbedToMedusa(String dir){
        dir=clean(dir);
        File f = new File(dir+File.separator);
        String[] children=f.list();
        Graph g;
        for (int i=0; i<children.length; i++){

            try {
                System.out.println(dir+File.separator+children[i]);
                g=dl.loadSimplest(children[i]);
                System.out.println("saving "+dir+File.separator+children[i]+".dat");
                dl.save(g,dir+File.separator+children[i]+".dat");
            } catch (IOException ex) {
                System.out.println(ex.getMessage());
            } catch (DataFormatException ex) {
                System.out.println(ex.getMessage());
            }
            
        }
        System.out.println("Converted as many files as possible in dir "+dir);
    }
    
    public void batchNormalize(String dir){
        
        
    }
    
    public void layoutAll(String dir){
        dir=clean(dir);
        File f = new File(dir);
        String[] children=f.list();
        Graph g;
        int dim=600;
        FRspring frSpring;
        for (int i=0; i<children.length; i++){
            if(children[i].endsWith(".dat")){
                try {
                    System.out.println("loading: "+children[i]);
                    g=dl.load(dir+File.separator+children[i]);
                    frSpring =
                            new FRspring(g,dim, dim);
                    frSpring.iterateAll();
                    // reset positions to 0-1
                    g.divideNodePosition(dim);
                    System.out.println("saving "+dir+File.separator+children[i]);
                    dl.save(g,dir+File.separator+children[i]);
                } catch (IOException ex) {
                    System.out.println(ex.getMessage());
                } catch (DataFormatException ex) {
                    System.out.println(ex.getMessage());
                }catch (java.lang.ArithmeticException ex){
                    System.out.println(ex.getMessage());
                }
            }
            
        }
        System.out.println("Converted as many files as possible in dir "+dir);
    }
    
    
    public void exportAllToPNG(String dir){
        dir=clean(dir);
        File f = new File(dir);
        String[] children=f.list();
        Graph g;
        FRspring frSpring;
        EditableGraphPanel dp = new EditableGraphPanel();
        for (int i=0; i<children.length; i++){
            if(children[i].endsWith(".dat")){
                try {
                    System.out.println("loading: "+children[i]);
                    g=dl.load(dir+File.separator+children[i]);
                    dp.setGraph(g);
                    System.out.println("saving "+dir+File.separator+children[i]);
                    dp.saveImage(dir+File.separator+children[i]+".png",1);
                    
                } catch (IOException ex) {
                    System.out.println(ex.getMessage());
                } catch (DataFormatException ex) {
                    System.out.println(ex.getMessage());
                }catch (java.lang.ArithmeticException ex){
                    System.out.println(ex.getMessage());
                }
            }
            
        }
        System.out.println("Converted as many files as possible in dir "+dir);
    }
    
    private String clean(String dir){
        // clean away the file separator, if present
        if(dir.endsWith(File.separator)){
            dir=dir.substring(0,dir.length()-1);
            System.out.println(dir);
        }
        return dir;
    }
    
    public static void main(String[] args) {
        BatchOperations b = new BatchOperations();
        System.out.println("Converting");
        b.convertTabbedToMedusa(args[0]);
        System.out.println("Laying out ");
        b.layoutAll(args[0]);
        //b.exportAllToPNG(args[0]);
    }
}
